package db.entity;

import db.util.Util;
import db.dto.StatisticDto;
import lombok.Data;

import java.text.ParseException;
import java.util.concurrent.atomic.AtomicLong;
import java.util.concurrent.atomic.LongAdder;

/**
 * 功能描述
 *
 * @author: scott
 * @date: 2022年12月28日 2:50 PM
 */
@Data
public class Statistic {
    long beginTime;
    LongAdder dataCount = new LongAdder();
    LongAdder dataCountTsdbSuccess = new LongAdder();
    LongAdder dataCountDiskSuccess = new LongAdder();

    public Statistic() {
        beginTime = System.currentTimeMillis();
    }

    public void incrDataCount() {
        this.dataCount.increment();
    }

    public void increDataCountTsdbSuccess() {
        this.dataCountTsdbSuccess.increment();
    }

    public void increDataCountTsdbSuccess(int count) {
        this.dataCountTsdbSuccess.add(count);
    }

    public void increDataCountDiskSuccess() {
        this.dataCountDiskSuccess.increment();
    }

    public void increDataCountDiskSuccess(int count) {
        this.dataCountDiskSuccess.add(count);
    }

    public double dataRate() {
        long time = System.currentTimeMillis() - this.beginTime;
        return dataCount.longValue() * 1000.0 / time;
    }

    public StatisticDto toDto() {
        StatisticDto statisticDto = new StatisticDto(this.beginTime, this.dataCount.longValue(),
                this.dataCountTsdbSuccess.longValue(), this.dataCountDiskSuccess.longValue());
        return statisticDto;
    }

    @Override
    public String toString() {
        long time = System.currentTimeMillis() - this.beginTime;
        double dataRate = dataCount.longValue() * 1000.0 / time;
        double dataCountTsdbSuccessRate = dataCountTsdbSuccess.longValue() * 1000.0 / time;
        double dataCountDiskSuccessRate = dataCountDiskSuccess.longValue() * 1000.0 / time;

        String beginTimeString = "";
        try {
            beginTimeString = Util.uTCMilliSecondsToDateString(this.beginTime);
        } catch (ParseException e) {
        }
        return "Statistic{" +
                "beginTime=" + beginTimeString +
                ", runningTime(ms)=" + time +
                ", runningTime(s)=" + time / 1000 +
                ", dataCount=" + dataCount +
                ", dataCountTsdbSuccess=" + dataCountTsdbSuccess +
                ", dataCountDiskSuccess=" + dataCountDiskSuccess +
                ", dataRate=" + dataRate +
                ", dataCountTsdbSuccessRate=" + dataCountTsdbSuccessRate +
                ", dataCountDiskSuccessRate=" + dataCountDiskSuccessRate +
                '}';
    }
}
