package db.netty;

import org.springframework.beans.factory.annotation.Autowired;

import db.entity.packages.BasePackage;
import db.entity.packages.DataResponsePackage;
import db.entity.packages.ListenResponsePackage;
import db.entity.packages.ScanResponsePackage;
import db.handler.PackageHandler;
import db.util.SpringUtils;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.ChannelHandler.Sharable;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Sharable
public class NettyServerHandler extends SimpleChannelInboundHandler<BasePackage> {

    private PackageHandler packageHandler;

    protected void processPackage(BasePackage basePackage) {
        if (basePackage instanceof DataResponsePackage) {
            packageHandler.handleDataResponsePackage((DataResponsePackage) basePackage);
        } else if (basePackage instanceof ScanResponsePackage) {
            packageHandler.handleScanResponsePackage((ScanResponsePackage) basePackage);
        } else if (basePackage instanceof ListenResponsePackage) {
            packageHandler.handleListenResponsePackage((ListenResponsePackage) basePackage);
        }
        // TODO 在这里扩展数据报文的处理
    }

    @Override
    protected void channelRead0(ChannelHandlerContext arg0, BasePackage basePackage) throws Exception {
        log.debug("channelRead0 收到数据:{}", basePackage);

        if (packageHandler == null) {
            packageHandler = SpringUtils.getBean(PackageHandler.class);
        }

        processPackage(basePackage);
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        log.error("NettyServerHandler exceptionCaught:{}", cause);
        cause.printStackTrace();
        ctx.fireExceptionCaught(cause);
    }
}
